<?php
/* Copyright (C) 2004-2014	Laurent Destailleur	<eldy@users.sourceforge.net>
 * Copyright (C) 2005-2012	Regis Houssin		<regis.houssin@capnetworks.com>
 * Copyright (C) 2008		Raphael Bertrand		<raphael.bertrand@resultic.fr>
 * Copyright (C) 2010-2014	Juanjo Menent		<jmenent@2byte.es>
 * Copyright (C) 2012		Christophe Battarel	<christophe.battarel@altairis.fr>
 * Copyright (C) 2012		Cédric Salvador		<csalvador@gpcsolutions.fr>
 * Copyright (C) 2012-2014	Raphaël Doursenaud	<rdoursenaud@gpcsolutions.fr>
 * Copyright (C) 2015		Marcos García		<marcosgdf@gmail.com>
 * Copyright (C) 2017		Ferran Marcet		<fmarcet@2byte.es>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

/**
 *	\file       htdocs/core/modules/facture/doc/pdf_crabe.modules.php
 *	\ingroup    facture
 *	\brief      File of class to generate customers invoices from crabe model
 */

require_once DOL_DOCUMENT_ROOT.'/core/modules/facture/modules_facture.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/company.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/functions2.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/pdf.lib.php';
ini_set('display_errors', 1);
error_reporting(E_ALL & ~E_NOTICE);


/**
 *	Class to manage PDF invoice template Crabe
 */
class pdf_invoice extends ModelePDFFactures
{
    var $db;
    var $name;
    var $description;
	var $update_main_doc_field;	// Save the name of generated file as the main doc when generating a doc with this template
    var $type;

    var $phpmin = array(4,3,0); // Minimum version of PHP required by module
    var $version = 'dolibarr';

    var $page_largeur;
    var $page_hauteur;
    var $format;
	var $marge_gauche;
	var	$marge_droite;
	var	$marge_haute;
	var	$marge_basse;

	var $emetteur;	// Objet societe qui emet

	/**
	 * @var bool Situation invoice type
	 */
	public $situationinvoice;

	/**
	 * @var float X position for the situation progress column
	 */
	public $posxprogress;


	/**
	 *	Constructor
	 *
	 *  @param		DoliDB		$db      Database handler
	 */
	function __construct($db)
	{
		global $conf,$langs,$mysoc;

		$langs->load("main");
		$langs->load("bills");

		$this->db = $db;
		$this->name = "INVOIC";
		$this->description = "Génération fichier INVOIC PMI";
		$this->update_main_doc_field = 1;		// Save the name of generated file as the main doc when generating a doc with this template

		// Dimensiont page
		$this->type = 'pdf';
		$formatarray=pdf_getFormat();
		$this->page_largeur = $formatarray['width'];
		$this->page_hauteur = $formatarray['height'];
		$this->format = array($this->page_largeur,$this->page_hauteur);
		$this->marge_gauche=isset($conf->global->MAIN_PDF_MARGIN_LEFT)?$conf->global->MAIN_PDF_MARGIN_LEFT:10;
		$this->marge_droite=isset($conf->global->MAIN_PDF_MARGIN_RIGHT)?$conf->global->MAIN_PDF_MARGIN_RIGHT:10;
		$this->marge_haute =isset($conf->global->MAIN_PDF_MARGIN_TOP)?$conf->global->MAIN_PDF_MARGIN_TOP:10;
		$this->marge_basse =isset($conf->global->MAIN_PDF_MARGIN_BOTTOM)?$conf->global->MAIN_PDF_MARGIN_BOTTOM:10;

		$this->option_logo = 1;                    // Affiche logo
		$this->option_tva = 1;                     // Gere option tva FACTURE_TVAOPTION
		$this->option_modereg = 1;                 // Affiche mode reglement
		$this->option_condreg = 1;                 // Affiche conditions reglement
		$this->option_codeproduitservice = 1;      // Affiche code produit-service
		$this->option_multilang = 1;               // Dispo en plusieurs langues
		$this->option_escompte = 1;                // Affiche si il y a eu escompte
		$this->option_credit_note = 1;             // Support credit notes
		$this->option_freetext = 1;				   // Support add of a personalised text
		$this->option_draft_watermark = 1;		   // Support add of a watermark on drafts

		$this->franchise=!$mysoc->tva_assuj;

		// Get source company
		$this->emetteur=$mysoc;
		if (empty($this->emetteur->country_code)) $this->emetteur->country_code=substr($langs->defaultlang,-2);    // By default, if was not defined

		// Define position of columns
		$this->posxdesc=$this->marge_gauche+1;
		if($conf->global->PRODUCT_USE_UNITS)
		{
			$this->posxtva=101;
			$this->posxup=118;
			$this->posxqty=135;
			$this->posxunit=151;
		}
		else
		{
			$this->posxtva=110;
			$this->posxup=126;
			$this->posxqty=145;
		}
		$this->posxdiscount=162;
		$this->posxprogress=126; // Only displayed for situation invoices
		$this->postotalht=174;
		if (! empty($conf->global->MAIN_GENERATE_DOCUMENTS_WITHOUT_VAT) || ! empty($conf->global->MAIN_GENERATE_DOCUMENTS_WITHOUT_VAT_COLUMN)) $this->posxtva=$this->posxup;
		$this->posxpicture=$this->posxtva - (empty($conf->global->MAIN_DOCUMENTS_WITH_PICTURE_WIDTH)?20:$conf->global->MAIN_DOCUMENTS_WITH_PICTURE_WIDTH);	// width of images
		if ($this->page_largeur < 210) // To work with US executive format
		{
		    $this->posxpicture-=20;
		    $this->posxtva-=20;
		    $this->posxup-=20;
		    $this->posxqty-=20;
		    $this->posxunit-=20;
		    $this->posxdiscount-=20;
		    $this->posxprogress-=20;
		    $this->postotalht-=20;
		}

		$this->tva=array();
		$this->localtax1=array();
		$this->localtax2=array();
		$this->atleastoneratenotnull=0;
		$this->atleastonediscount=0;
		$this->situationinvoice=False;
	}


	/**
     *  Function to build pdf onto disk
     *
     *  @param		Object		$object				Object to generate
     *  @param		Translate	$outputlangs		Lang output object
     *  @param		string		$srctemplatepath	Full path of source filename for generator using a template file
     *  @param		int			$hidedetails		Do not show line details
     *  @param		int			$hidedesc			Do not show desc
     *  @param		int			$hideref			Do not show ref
     *  @return     int         	    			1=OK, 0=KO
	 */
	function write_file($object,$outputlangs,$srctemplatepath='',$hidedetails=0,$hidedesc=0,$hideref=0)
	{
		global $user,$langs,$conf,$mysoc,$db,$hookmanager,$nblignes;


		$outputlangs->load("main");
		$outputlangs->load("dict");
		$outputlangs->load("companies");
		$outputlangs->load("bills");
		$outputlangs->load("products");
		$version="dolibarr";

		$fichier=fopen ("/var/www/$version/documents/facture/$object->ref/$object->ref.txt","w+");
		
		$idmessage=substr($object->ref,2,4);
		$idmessage.=substr($object->ref,7,4);
		$datemessage=date("YmdHi", filemtime("/var/www/$version/documents/facture/$object->ref/$object->ref.txt"));
		$signe="+";
		
		// If BILLING contact defined on invoice, we use it
		$usecontact=false;
		$arrayidcontact=$object->getIdContact('external','BILLING');
		if (count($arrayidcontact) > 0)
		{
			$usecontact=true;
			$result=$object->fetch_contact($arrayidcontact[0]);
		}

		//Recipient name
		// On peut utiliser le nom de la societe du contact
		if ($usecontact && !empty($conf->global->MAIN_USE_COMPANY_NAME_OF_CONTACT)) {
			$thirdparty = $object->contact;
		} else {
			$thirdparty = $object->thirdparty;
		}
		
		// If BILLING contact defined on invoice, we use it
		$usecontact=false;
		$arrayidcontact=$object->getIdContact('external','SHIPPING');
		if (count($arrayidcontact) > 0)
		{
			$usecontact=true;
			$result=$object->fetch_contact($arrayidcontact[0]);
		}

		//Recipient name
		// On peut utiliser le nom de la societe du contact
		if ($usecontact && !empty($conf->global->MAIN_USE_COMPANY_NAME_OF_CONTACT)) {
			$thirdparty_shipping = $object->contact;
		} else {
			$thirdparty_shipping = $object->thirdparty;
		}
		
	//echo "<pre>";print_r($thirdparty_shipping);
			
		
		
		
		
		//Date et réference facture
		$texte= "SER   ACOBAL                                               CEGIDPMI                                                            INVOICD  96AUNA14051\r\n";
		$object->ref=str_pad($object->ref,17);
		$texte.="T1400 380                       ".$object->ref." ".dol_print_date($object->date,"%Y%m%d",false,$outputlangs)."\r\n";
		$texte.="Z0201 REGACOBAL                                                                                                                                      +000000000000000EUR                                                   B 408768679 Saint-Etienne                                                                                                                   \r\n";

		//Adresse de facturation
		//Aller chercher clef edi tiers payeur si présent voir le if au dessus 
	
/*      if ($thirdparty->socname=="PCA ESPAGNA") 
		$texte.="C0100 0060462004003          92 ".str_pad($thirdparty->socname,178).str_pad($thirdparty->lastname." ".$thirdparty->address,71).$thirdparty->zip." ".str_pad($thirdparty->town,29).str_pad($object->thirdparty->country,52).str_pad($object->thirdparty->tva_intra,34).$object->thirdparty->country_code."\r\n";

		else 
		$texte.="C0100 0002542065479          92 ".str_pad($thirdparty->socname,178).str_pad($thirdparty->lastname." ".$thirdparty->address,71).$thirdparty->zip." ".str_pad($thirdparty->town,29).str_pad($object->thirdparty->country,52).str_pad($object->thirdparty->tva_intra,34).$object->thirdparty->country_code."\r\n";
*/
		
		$texte.="C0100 ".str_pad(substr($object->thirdparty->array_options['options_clefedi'],0,13),23)."92 ".str_pad($thirdparty->socname,178).str_pad($thirdparty->lastname." ".$thirdparty->address,71).$thirdparty->zip." ".str_pad($thirdparty->town,29).str_pad($object->thirdparty->country,52).str_pad($object->thirdparty->tva_intra,34).$object->thirdparty->country_code."\r\n";

		
		//ACOBAL
		$texte.="C1800 0931408768679          92 ACOBAL                             Saint Chamond Cedex                42403                              Z.I. du Clos Marquet               CS 10 177                             Z.I. du Clos Marquet               CS 10 177                          42403 Saint Chamond Cedex          FRANCE                             ".str_pad($object->thirdparty->array_options['options_code_va'],17)."FR39408768679                     FR\r\n";
		
		//Adresse de facturation ?
/*		if ($thirdparty->socname=="PCA ESPAGNA") 
		$texte.="C1200 0060462004003          92 ".str_pad($thirdparty->socname,178).str_pad($thirdparty->lastname." ".$thirdparty->address,71).$thirdparty->zip." ".str_pad($thirdparty->town,29).str_pad($object->thirdparty->country,52).str_pad($object->thirdparty->tva_intra,34).$object->thirdparty->country_code."\r\n";

		else 
		$texte.="C1200 0002542065479          92 ".str_pad($thirdparty->socname,178).str_pad($thirdparty->lastname." ".$thirdparty->address,71).$thirdparty->zip." ".str_pad($thirdparty->town,29).str_pad($object->thirdparty->country,52).str_pad($object->thirdparty->tva_intra,34).$object->thirdparty->country_code."\r\n";
*/		
		$texte.="C1200 ".str_pad(substr($object->thirdparty->array_options['options_clefedi'],0,13),23)."92 ".str_pad($thirdparty->socname,178).str_pad($thirdparty->lastname." ".$thirdparty->address,71).$thirdparty->zip." ".str_pad($thirdparty->town,29).str_pad($object->thirdparty->country,52).str_pad($object->thirdparty->tva_intra,34).$object->thirdparty->country_code."\r\n";

		//Monnaie
		$texte.="F0400                      EUR4  EUR11\r\n";
		
		//Date de reglement
		$texte.="F1101                                                                                                                                                                                                  1                                            ".dol_print_date($object->date_lim_reglement,"%Y%m%d",false,$outputlangs)."\r\n";
		
		//Boucle produit
		
		//$prod = new Product($this->db);
		//echo "<pre>";print_r($object);
		$nblines = count($object->lines);
		
		$minus_rang=0;
		for ($i = 0 ; $i < $nblines ; $i++) { 
		
			if ($object->lines[$i]->product_type==9) {
				
				//$texte.=$object->lines[$i]->desc;
				preg_match("# / (.*)#",$object->lines[$i]->desc,$cmd_client);
				
				//echo "<pre>";print_r ($object->lines[$i]->desc);
				preg_match("#Livraison (.*)#",$object->lines[$i]->desc,$bl);
				//$num_bl=substr(str_replace("-","",$bl[1]),0,11);
				$num_bl=substr($bl[1],0,12);
				$cmd_clientok=substr($cmd_client[1],0,8);
				//$texte.="/////".$cmd_clientok;
			}
			else {
				$produit = new Product($this->db);
				$result=$produit->fetch($object->lines[$i]->fk_product);
				$cmd_client=$object->ref_client;
				//Le poids est dans $produit->weight
				//echo "<pre>";print_r($produit);
			
				//REfs(aller chercher $object->lines[$i]->fk_product), designation, poids (aller chercher $object->lines[$i]->fk_product) produits
				//Poids de la ligne 
			
				$sql = "SELECT ref FROM ".MAIN_DB_PREFIX."custom_product_ref WHERE fk_soc = $object->socid AND fk_product= ".$object->lines[$i]->fk_product;
				$res = $this->db->query($sql);
				if($res) {
					$obj = $this->db->fetch_object($res);
					$minus_rang=-1;
				
				}
				$poids_ligne=$produit->weight*$object->lines[$i]->qty;
			
				$texte.="A0100 ".str_pad($object->lines[$i]->rang+$minus_rang,6,"0",STR_PAD_LEFT)."     ".str_pad($obj->ref,38).str_pad($object->lines[$i]->ref,114).str_pad($object->lines[$i]->libelle,258)."FR                      +".str_pad($poids_ligne,13,"0",STR_PAD_LEFT)."   KGM\r\n";

				//Qte Produit
				$texte.=str_pad("A0400",268)."+".str_pad($object->lines[$i]->qty*1000,10,"0",STR_PAD_LEFT)."PCE\r\n";
			
				//prix total de la ligne
			
				$texte.="F0903    ".$signe.str_pad($object->lines[$i]->multicurrency_total_ht*100,15,"0",STR_PAD_LEFT)."\r\n";
				//prix pour mille piece  
				$texte.=str_pad("F0300",87).$signe.str_pad($object->lines[$i]->multicurrency_subprice*100000,17,"0",STR_PAD_LEFT)."+00001000PCE\r\n";
				//ref cmd client
				$texte.="E0503 ON $cmd_clientok\r\n";
				//Num BL ? date bl ?
				$sql = "SELECT date_creation FROM `llx_livraison` WHERE `ref` LIKE '".$bl[1]."'";
				$res = $this->db->query($sql);
				if($res) {
					$obj = $this->db->fetch_object($res);
				
				}
				//date(Ymd,$obj->date_creation)
				$date = date_create($obj->date_creation);
				$date= date_format($date, 'Ymd');
				$texte.="E0503 AAK".str_pad($num_bl,33).$date."\r\n";
				
				//Taux TVA Ligne
				//Ici pour non TVA
				$texte.="F0701    VAT+". str_pad($object->lines[$i]->tva_tx*100,9,"0",STR_PAD_LEFT)  ."\r\n";
				//Adresse Site livraison 
			
				$texte.="C0500 ".str_pad($object->thirdparty->array_options['options_clefedi'],23)."92 ".str_pad($thirdparty_shipping->lastname,178).str_pad($thirdparty_shipping->address,70).$thirdparty_shipping->zip." ".str_pad($thirdparty_shipping->town,29)."FRANCE                                              FR \r\n";

				//Porte livraison
				$texte.="H0200 $thirdparty_shipping->firstnwame \r\n"; 
			} 	
		}
		
		//Entete facture
		
		
		//Taux TVA + Total TVA et HT
		
		$total_tva=str_pad($object->total_tva*1000,15,"0",STR_PAD_LEFT);
		$total_ht=str_pad($object->total_ht*1000,15,"0",STR_PAD_LEFT);
		$total_ttc=str_pad($object->total_ttc*1000,15,"0",STR_PAD_LEFT);
		//Ici pour non TVA
		if ($object->thirdparty->tva_assuj==0) {
			$texte.="F0703 061VAT+000000000\r\n";	
		}
		else $texte.="F0703 200VAT+000020000                       ".$signe.$total_tva.$signe.$total_ht."\r\n";
		
		
		//Total HT
		$texte.="F0906 79 ".$signe.$total_ht."\r\n";
	
		//Total TTC
		$texte.="F0906 77 ".$signe.$total_ttc."\r\n";
		
		//Totalt HT +port
		$texte.="F0906 125".$signe.$total_ht."\r\n";
		
		//Montant TVA
		$texte.="F0906 176".$signe.$total_ttc."\r\n";
		
		//Montant ??
		$texte.="F0906 136+000000000000000\r\n";
		$texte.="F0906 109+000000000000000\r\n";
		
		//SI tout ok modifier dans llx_facture le champ model_pdf mettre à crabe (modèle par defaut)
		$sql="UPDATE `llx_facture` SET `model_pdf`='neopost' WHERE ref='$object->ref'";
		
		$res = $this->db->query($sql);
		
		fwrite($fichier,$texte);
		fclose($fichier);
		$object->ref=str_replace(" ","",$object->ref);
		if (!copy("/var/www/$version/documents/facture/$object->ref/$object->ref.txt","/home/acobal/EDI/OUT/$object->ref.txt")) {
			echo "La copie du fichier $filename dans OUT a échoué...\n";
		
		}
		chmod ("/home/acobal/EDI/OUT/$object->ref.txt",0666);
		
	
		
		
		return 1;

	
	}


	
}

