<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/zones/class/html.form.zones.class.php
 *  \ingroup    zones
 *  \brief      File of class to manage form for zones
 */
dol_include_once("/fraistransport/class/zone.class.php");


class ZonesForm
{
    var $db;
    var $error;

    /**
     * Constructor
     * @param      $DB      Database handler
     */
    function ZonesForm($DB)
    {
        $this->db = $DB;
    }

    /**
     *    Return combo list of products sets
     *    @param     selected         Id preselected products set
     *    @param     htmlname         Name of html select object
     *    @param     htmloption       Options html on select object
     *    @return    string           HTML string with select
     */
    function select_zones($selected='',$htmlname='fk_zone', $htmloption='', $empty = false)
    {
        global $conf, $langs;

        
        $zone = new Zone($this->db);

        $zones = $zone->getZones();
                
        //Build select
        $select = '<select class="flat" id = "'.$htmlname.'" name = "'.$htmlname.'" '.$htmloption.'>';
        if ($empty)
        {
            $select .= '<option value="0" '.(empty($selected) ? 'selected="selected"' : '').'>&nbsp;</option>';

        }
        if (is_array($zones) && sizeof($zones))
        {
            foreach ($zones as $id => $zone)
            {
                $select .= '<option value="'.$id.'" '.($id == $selected ? 'selected="selected"' : '').'>'.$zone->label.'</option>';
            }            
        }

        
        $select .= '</select>';

        return $select;
    }

    /**
     *   Retourne la liste deroulante des regions actives dont le pays est actif
     *   La cle de la liste est le code (il peut y avoir plusieurs entree pour
     *   un code donnee mais dans ce cas, le champ pays et lang differe).
     *   Ainsi les liens avec les regions se font sur une region independemment de son name.
     *
     *   @param     string      $selected       Preselected value
     *   @param     string      $htmlname       Name of HTML select field
     *   @return    void
     */
    function select_region($selected='',$htmlname='region_id')
    {
        global $conf,$langs;
        $langs->load("dict");
        $out = '';

        $sql = "SELECT r.rowid, r.nom as label, r.active, r.code_region, c.code as country_code, c.label as country";
        $sql.= " FROM ".MAIN_DB_PREFIX."c_regions as r, ".MAIN_DB_PREFIX."c_country as c";
        $sql.= " WHERE r.fk_pays=c.rowid AND r.active = 1 and c.active = 1";
        $sql.= " ORDER BY c.label, r.nom ASC";

        dol_syslog(get_class($this)."::select_region", LOG_DEBUG);
        $resql=$this->db->query($sql);
        if ($resql)
        {
            $out .= '<select style="width: 300px" class="flat" id="'.$htmlname.'" name="'.$htmlname.'">';
            $num = $this->db->num_rows($resql);
            $i = 0;
            if ($num)
            {
                $country='';
                while ($i < $num)
                {
                    $obj = $this->db->fetch_object($resql);

                    if ($obj->code_region == 0) {
                        $out .= '<option value="0">&nbsp;</option>';
                    }
                    else {
                        if ($country == '' || $country != $obj->country)
                        {
                            // Show break
                            $key=$langs->trans("Country".strtoupper($obj->country_code));
                            $valuetoshow=($key != "Country".strtoupper($obj->country_code))?$obj->country_code." - ".$key:$obj->country;
                            $out .= '<option value="-1" disabled>----- '.$valuetoshow." -----</option>\n";
                            $country=$obj->country;
                        }

                        if ($selected > 0 && $selected == $obj->rowid)
                        {
                            $out .= '<option value="'.$obj->rowid.'" selected>'.$obj->label.'</option>';
                        }
                        else
                        {
                            $out .= '<option value="'.$obj->rowid.'">'.$obj->label.'</option>';
                        }
                    }
                    $i++;
                }
            }
            $out .= '</select>';
        }
        else
        {
            dol_print_error($this->db);
        }

        // Make select dynamic
        include_once DOL_DOCUMENT_ROOT . '/core/lib/ajax.lib.php';
        $out .= ajax_combobox($htmlname);

        return $out;

    }

    /**
     *   Retourne la liste deroulante des regions actives dont le pays est actif
     *   La cle de la liste est le code (il peut y avoir plusieurs entree pour
     *   un code donnee mais dans ce cas, le champ pays et lang differe).
     *   Ainsi les liens avec les regions se font sur une region independemment de son name.
     *
     *   @param     string      $selected       Preselected value
     *   @param     string      $htmlname       Name of HTML select field
     *   @return    void
     */
    function select_country($selected='',$htmlname='country_id')
    {
        global $conf,$langs;
        $langs->load("dict");
        $out = '';

		$sql = "SELECT rowid, code as code_iso, code_iso as code_iso3, label, favorite";
		$sql.= " FROM ".MAIN_DB_PREFIX."c_country";
		$sql.= " WHERE active > 0";
        $sql.= " ORDER BY label ASC";

        dol_syslog(get_class($this)."::select_region", LOG_DEBUG);
        $resql=$this->db->query($sql);
        if ($resql)
        {
            $out .= '<select class="flat" id="'.$htmlname.'" name="'.$htmlname.'">';
            $out .= '<option value="0">&nbsp;</option>';

            $num = $this->db->num_rows($resql);
            $i = 0;
            if ($num)
            {
                while ($i < $num)
                {
                    $obj = $this->db->fetch_object($resql);

                    if ($selected > 0 && $selected == $obj->rowid)
                    {
                        $out .= '<option value="'.$obj->rowid.'" selected>'.$obj->label.'</option>';
                    }
                    else
                    {
                        $out .= '<option value="'.$obj->rowid.'">'.$obj->label.'</option>';
                    }

                    $i++;
                }
            }
            $out .= '</select>';
        }
        else
        {
            dol_print_error($this->db);
        }

        // Make select dynamic
        include_once DOL_DOCUMENT_ROOT . '/core/lib/ajax.lib.php';
        $out .= ajax_combobox($htmlname);

        return $out;

    }
}

?>
