<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/fraistransport/class/fraistransport.class.php
 *  \ingroup    fraistransport
 *  \brief      File of class to manage predefined products sets
 */
require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';


/**
 * Class to manage products or services
 */
class Transporter extends CommonObject
{
	public $element='transporteurs';
	public $table_element='c_transporteurs';
	public $fk_element='fk_transporteur';
	public $picto = 'generic';
	public $ismultientitymanaged = 1;	// 0=No test on entity, 1=Test with field entity, 2=Test with link by societe

	/**
	 * {@inheritdoc}
	 */
	protected $table_ref_field = 'rowid';


	/**
	 * Transporter label
	 * @var string
	 */
	public $label;

	/**
	 * Transporter ref
	 * @var string
	 */
	public $ref;

	/**
     * Product id link
     * @var string
     */
	public $fk_product;


	/**
     * Transporter id
     * @var string
     */
	public $id = 0;

	/**
     * Behavior if out of range
     * @var string
     */
	public $out_of_range;

	/**
     * Is this transporter subject to the fuel tax ?
     * @var string
     */
	public $subject_to_fuel_tax;

    /**
     * Shipping method id
     * @var int
     */
    public $fk_shipping_method = 0;

	/**
     * Active
     * @var int
     */
	public $active = 1;

	/**
	 * Do not retain this transporter if outside range
	 */
	const DO_NOT_TAKE = 0;

	/**
	 * Take min for this transporter if outside range
	 */
	const TAKE_MIN = 1;

	/**
	 * Take max for this transporter if outside range
	 */
	const TAKE_MAX = 2;

	/**
	 * Take max for this transporter if outside range
	 */
	const FORCE_TO_ZERO = 3;

	/**
	 *  Constructor
	 *
	 *  @param      DoliDB		$db      Database handler
	 */
	function __construct($db)
	{
		global $langs;

		$this->db = $db;
	}

	/**
	 *    Check that ref and label are ok
	 *
	 *    @return     int         >1 if OK, <=0 if KO
	 */
	function check()
	{

		$this->ref = dol_sanitizeFileName(stripslashes($this->ref));

		$err = 0;
		if (dol_strlen(trim($this->ref)) == 0)
		$err++;

		if (dol_strlen(trim($this->label)) == 0)
		$err++;

		if ($err > 0)
		{
			return 0;
		}
		else
		{
			return 1;
		}
	}

    /**
     *    Check properties of product are ok (like name, barcode, ...).
     *    All properties must be already loaded on object (this->barcode, this->barcode_type_code, ...).
     *
     *    @return     int		0 if OK, <0 if KO
     */
    function verify()
    {
        $this->errors=array();

        $result = 0;
        $this->ref = trim($this->ref);

        if (! $this->ref)
        {
            $this->errors[] = 'ErrorBadRef';
            $result = -2;
        }

        /*$this->label = trim($this->label);

        if (! $this->label)
        {
            $this->errors[] = 'ErrorBadLabel';
            $result = -2;
        }*/

        return $result;
    }

	/**
	 *	Insert product into database
	 *
	 *	@param	User	$user     		User making insert
	 *  @param	int		$notrigger		Disable triggers
	 *	@return int			     		Id of product/service if OK, < 0 if KO
	 */
	function create($user,$notrigger=0)
	{
		global $conf, $langs, $mysoc;

        	$error=0;

		$this->ref = dol_string_nospecial(trim($this->ref));
		$this->label = trim($this->label);

		// Check parameters
		if ($this->fk_shipping_method <= 0)
		{
			$this->error='ErrorMandatoryParametersNotProvided';
			return -1;
		}

		// Check parameters
		if (empty($this->ref))
		{
			$this->error='ErrorMandatoryParametersNotProvided';
			return -1;
		}
		
		dol_syslog(get_class($this)."::create", LOG_DEBUG);

		$now=dol_now();

		$this->db->begin();

		// Check more parameters
		// If error, this->errors[] is filled
		$result = $this->verify();

		if ($result >= 0)
		{

			$sql = "SELECT rowid as fk_product";
			$sql.= " FROM ".MAIN_DB_PREFIX."product";
			$sql.= " WHERE entity IN (".getEntity('product').")";
			$sql.= " AND ref = '" .$this->db->escape($this->ref)."'";

			$result = $this->db->query($sql);
			if ($result)
			{
				$num = $this->db->num_rows($result);

				if ($num > 0)
				{
					$obj = $this->db->fetch_object($result);
					$this->fk_product = $obj->fk_product;
				}
				else
				{
					$tva_tx_txt = get_default_tva($mysoc, $mysoc);
					
					$tva_tx = preg_replace('/[^0-9\.].*$/', '', $tva_tx_txt);     // keep remove all after the 

					$npr = preg_match('/\*/', $tva_tx_txt) ? 1 : 0;

			        $product_label = $conf->global->TRANSPORTER_PRODUCT_LABEL .' '.$this->label;
			        

					// Create product
					$product = new Product($this->db);
					$product->ref                   = $this->ref;
		            $product->label                 = $product_label;
		            $product->price_base_type       = 'HT';
			        $product->tva_tx = $tva_tx;
			        $product->tva_npr = $npr;
		            $product->type               	 = Product::TYPE_PRODUCT;
		            $product->description        	 = '';

		            $fk_product = $product->create($user);

		            if ($fk_product < 0)
		            {
		            	$this->error = $product->error;
		            	return -1;
		            }

		            $this->fk_product = $fk_product;
				}

				$sql = "INSERT INTO ".MAIN_DB_PREFIX."c_transporteurs (";
				$sql.= " datec";
                $sql.= ", tms";
				$sql.= ", entity";
				$sql.= ", out_of_range";
				$sql.= ", subject_to_fuel_tax";
				$sql.= ", fk_product";
				$sql.= ", fk_shipping_method";
				$sql.= ", label";
				$sql.= ", ref";
				$sql.= ") VALUES (";
				$sql.= "'".$this->db->idate($now)."'";
                $sql.= ", '".$this->db->idate($now)."'";
				$sql.= ", ".$conf->entity;
				$sql.= ", ".$this->out_of_range;
				$sql.= ", ".$this->subject_to_fuel_tax;
				$sql.= ", ".$this->fk_product;
                $sql.= ", ".$this->fk_shipping_method;
				$sql.= ", ".(! empty($this->label)?"'".$this->db->escape($this->label)."'":"null");
				$sql.= ", ".(! empty($this->ref)?"'".$this->db->escape($this->ref)."'":"null");
				$sql.= ")";

				dol_syslog(get_class($this)."::Create", LOG_DEBUG);
				$result = $this->db->query($sql);
				if ( $result )
				{
					$id = $this->db->last_insert_id(MAIN_DB_PREFIX."c_transporteurs");

					if ($id > 0)
					{
						$this->id = $id;

					}
					else
					{
						$error++;
						$this->error='ErrorFailedToGetInsertedId';
					}
				}
				else
				{
					$error++;
					$this->error=$this->db->lasterror();
				}
			}
			else
			{
				$error++;
			    $this->error=$this->db->lasterror();
			}				

			if (! $error)
			{
				$this->db->commit();
				return $this->id;
			}
			else
			{
				$this->db->rollback();
				return -$error;
			}
        }
        else
       {
            $this->db->rollback();
            dol_syslog(get_class($this)."::Create fails verify ".join(',',$this->errors), LOG_WARNING);
            return -3;
        }

	}

	/**
	 *	Update a record into database.
	 *
	 *	@param	int		$id         Id of product set
	 *	@param  User	$user       Object user making update
	 *	@param	int		$notrigger	Disable triggers
	 *	@param	string	$action		Current action for hookmanager ('add' or 'update')
	 *	@return int         		1 if OK, -1 if ref already exists, -2 if other error
	 */
	function update($id, $user, $notrigger=false, $action='update')
	{
		global $langs, $conf, $hookmanager, $mysoc;

		$error=0;

		// Check parameters
		if (! $this->label) $this->label = 'MISSING LABEL';

		// Clean parameters
		$this->ref = dol_string_nospecial(trim($this->ref));
		$this->label = trim($this->label);
		


        $this->db->begin();

        // Check name is required and codes are ok or unique.
        // If error, this->errors[] is filled
        $result = 0;
        if ($action != 'add')
        {
        	$result = $this->verify();	// We don't check when update called during a create because verify was already done
        }

        if ($result >= 0)
        {
			$sql = "SELECT rowid as fk_product";
			$sql.= " FROM ".MAIN_DB_PREFIX."product";
			$sql.= " WHERE entity IN (".getEntity('product').")";
			$sql.= " AND ref = '" .$this->db->escape($this->ref)."'";

			$result = $this->db->query($sql);
			if ($result)
			{
				$num = $this->db->num_rows($result);

				if ($num > 0)
				{
					$obj = $this->db->fetch_object($result);
					$this->fk_product = $obj->fk_product;

					/*
					$product_label = $conf->global->TRANSPORTER_PRODUCT_LABEL .' '.$this->label;

					$productstatic = new Product($this->db);
					$productstatic->fetch($this->fk_product);
		
					$productstatic->ref = $this->ref;
					$productstatic->label = $product_label;
					$result = $productstatic->update($this->fk_product, $user);
					*/
				}
				else
				{
					$tva_tx_txt = get_default_tva($mysoc, $mysoc);
					
					$tva_tx = preg_replace('/[^0-9\.].*$/', '', $tva_tx_txt);     // keep remove all after the 

					$npr = preg_match('/\*/', $tva_tx_txt) ? 1 : 0;

			        $product_label = $conf->global->TRANSPORTER_PRODUCT_LABEL .' '.$this->label;
			        

					// Create product
					$product = new Product($this->db);
					$product->ref                   = $this->ref;
		            $product->label                 = $product_label;
		            $product->price_base_type       = 'HT';
			        $product->tva_tx = $tva_tx;
			        $product->tva_npr = $npr;
		            $product->type               	 = Product::TYPE_PRODUCT;
		            $product->description        	 = '';

		            $fk_product = $product->create($user);

		            if ($fk_product < 0)
		            {
		            	$this->error = $product->error;
		            	return -1;
		            }

		            $this->fk_product = $fk_product;

				}
			}

			if ($result >= 0)
			{
				$sql = "UPDATE ".MAIN_DB_PREFIX."c_transporteurs";
				$sql.= " SET label = '" . $this->db->escape($this->label) ."'";
				$sql.= ", subject_to_fuel_tax = " . $this->subject_to_fuel_tax;
				$sql.= ", out_of_range = " . $this->out_of_range;
				$sql.= ", fk_product = " . $this->fk_product;
                $sql.= ", fk_shipping_method = " . $this->fk_shipping_method;
				$sql.= ", ref = '" . $this->db->escape($this->ref) ."'";
				$sql.= " WHERE rowid = " . $id;

				dol_syslog(get_class($this)."::update", LOG_DEBUG);

				$resql=$this->db->query($sql);
				if ($resql)
				{
					$this->id = $id;

					$this->db->commit();
					return 1;
				}
				else
				{
					$this->error=$langs->trans("Error")." : ".$this->db->error()." - ".$sql;
					$this->errors[]=$this->error;
					$this->db->rollback();

					return -2;				
				}           			
			}
			else
			{
				$error++;
				$this->error = $productstatic->error;           			
			}


			if (! $error)
			{
				$this->db->commit();
				return $this->id;
			}
			else
			{
				$this->db->rollback();
				return -$error;
			}

        }
        else
        {
            $this->db->rollback();
            dol_syslog(get_class($this)."::Update fails verify ".join(',',$this->errors), LOG_WARNING);
            return -3;
        }
	}

	/**
	 *  Delete a product set from database (if not used)
	 *
	 *	@param      User	$user       Product id (usage of this is deprecated, delete should be called without parameters on a fetched object)
	 *  @param      int     $notrigger  Do not execute trigger
	 * 	@return		int					< 0 if KO, 0 = Not possible, > 0 if OK
	 */
	function delete(User $user, $notrigger=0)
	{
		global $conf, $langs;

		$error=0;

		// Check parameters
		if (empty($this->id))
		{
			$this->error = "Object must be fetched before calling delete";
			return -1;
		}
		
		$this->db->begin();

		$productstatic = new Product($this->db);
		$productstatic->fetch($this->fk_product);

		$result = $productstatic->delete($user);

		if ($result < 0)
		{
			$error++;
			$this->error = $productstatic->error;
		}

		// Delete product
		if (! $error)
		{
			$sqlz = "DELETE FROM ".MAIN_DB_PREFIX."c_tranches_poids_departements";
			$sqlz.= " WHERE fk_transporteur = ".$this->id;
			dol_syslog(get_class($this).'::delete', LOG_DEBUG);
			$resultz = $this->db->query($sqlz);

			if ( ! $resultz )
			{
				$error++;
				$this->errors[] = $this->db->lasterror();
			}

			$sqlz = "DELETE FROM ".MAIN_DB_PREFIX."c_tranches_poids_regions";
			$sqlz.= " WHERE fk_transporteur = ".$this->id;
			dol_syslog(get_class($this).'::delete', LOG_DEBUG);
			$resultz = $this->db->query($sqlz);

			if ( ! $resultz )
			{
				$error++;
				$this->errors[] = $this->db->lasterror();
			}

			$sqlz = "DELETE FROM ".MAIN_DB_PREFIX."c_transporteurs";
			$sqlz.= " WHERE rowid = ".$this->id;
			dol_syslog(get_class($this).'::delete', LOG_DEBUG);
			$resultz = $this->db->query($sqlz);
			
			if ( ! $resultz )
			{
				$error++;
				$this->errors[] = $this->db->lasterror();
			}

		}

		

		if (! $error)
		{
			$this->db->commit();
			return 1;
		}
		else
		{
			foreach($this->errors as $errmsg)
			{
				dol_syslog(get_class($this)."::delete ".$errmsg, LOG_ERR);
				$this->error.=($this->error?', '.$errmsg:$errmsg);
			}
			$this->db->rollback();
			return -$error;
		}

	}

	/**
	 *  Load a product set in memory from database
	 *
	 *  @param	int		$id      			Id of product/service to load
	 *  @param  string	$ref     			Ref of product/service to load
	 *  @return int     					<0 if KO, 0 if not found, >0 if OK
	 */
	function fetch($id='', $ref='')
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::fetch id=".$id." ref=".$ref);

		// Check parameters
		if (! $id && ! $ref)
		{
			$this->error='ErrorWrongParameters';
			dol_print_error(get_class($this)."::fetch ".$this->error);
			return -1;
		}

		$sql = "SELECT t.rowid, t.ref, t.label, sm.libelle as shipment_mode_label, t.fk_product, t.fk_shipping_method, t.out_of_range, t.subject_to_fuel_tax, t.active, t.datec, t.tms";
		$sql.= " FROM ".MAIN_DB_PREFIX."c_transporteurs t";
        $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_shipment_mode sm ON sm.rowid = t.fk_shipping_method";
		if ($id) $sql.= " WHERE t.rowid = ".$this->db->escape($id);
		else
		{
			$sql.= " WHERE t.entity IN (".getEntity($this->element).")";
			if ($ref) $sql.= " AND t.ref = '".$this->db->escape($ref)."'";
		}

		$resql = $this->db->query($sql);
		if ( $resql )
		{
			if ($this->db->num_rows($resql) > 0)
			{
				$obj = $this->db->fetch_object($resql);

				$this->id							= $obj->rowid;
				$this->ref							= $obj->ref;
                $this->label                        = !empty($obj->shipment_mode_label) ? $obj->shipment_mode_label : $obj->label;
				$this->subject_to_fuel_tax			= $obj->subject_to_fuel_tax;
				$this->fk_product					= $obj->fk_product;
				$this->fk_shipping_method           = $obj->fk_shipping_method;
				$this->active						= $obj->active;

				$this->date_creation				= $obj->datec;
				$this->date_modification			= $obj->tms;
				
				$this->entity						= $obj->entity;

				$this->db->free($resql);

				return 1;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			dol_print_error($this->db);
			return -1;
		}
	}	

	/**
	 *  Load transporters
	 *
	 *  @return int     					<0 if KO, 0 if not found, >0 if OK
	 */
	function getTransporters($active = 0)
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::getTransporters");

		$transporteurs = array();

        $sql = "SELECT t.rowid, t.ref, t.label, sm.libelle as shipment_mode_label, t.fk_product, t.fk_shipping_method, t.out_of_range, t.subject_to_fuel_tax, t.active, t.datec, t.tms";
        $sql.= " FROM ".MAIN_DB_PREFIX."c_transporteurs t";
        $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_shipment_mode sm ON sm.rowid = t.fk_shipping_method";
		$sql.= " WHERE t.entity IN (".getEntity($this->element).")";
		$sql.= $active ? " AND t.active = ".$active : "";

		$resql = $this->db->query($sql);
		if ( $resql )
		{
			if ($this->db->num_rows($resql) > 0)
			{
				while ($obj = $this->db->fetch_object($resql))
				{
				    $obj->label = !empty($obj->shipment_mode_label) ? $obj->shipment_mode_label : $obj->label;
					$transporteurs[$obj->rowid]			= $obj;				
				}

				$this->db->free($resql);

				return $transporteurs;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			dol_print_error($this->db);
			return -1;
		}
	}
}
