<?php
/* Copyright (C) 2004-2017 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017 Mikael Carlavan <contact@mika-carl.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  \file       htdocs/fraistransport/class/fraistransport.class.php
 *  \ingroup    fraistransport
 *  \brief      File of class to manage predefined products sets
 */
require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';

dol_include_once("/fraistransport/class/weighttool.class.php");

/**
 * Class to manage products or services
 */
class WeightRange extends CommonObject
{
	public $element='weightranges';
	public $table_element='';
	public $fk_element='fk_weightrange';
	public $picto = 'generic';
	public $ismultientitymanaged = 1;	// 0=No test on entity, 1=Test with field entity, 2=Test with link by societe

	/**
	 * {@inheritdoc}
	 */
	protected $table_ref_field = 'rowid';



	/**
     * Transporter id link
     * @var string
     */
	public $fk_transporteur = 0;

	/**
     * Zone id link
     * @var string
     */
	public $fk_zone = 0;

	/**
     * Min weight
     * @var string
     */
	public $poids_min = 0;

	/**
     * Max weight
     * @var string
     */
	public $poids_max = 0;

	/**
     *  Min weight unit
     * @var string
     */
	public $fk_unit_min = 0;

	/**
     *  Max weight unit
     * @var string
     */
	public $fk_unit_max = 0;

    /**
     * Poids reel
     * @var string
     */
    public $poids_reel = 1;

	/**
     * Amount
     * @var string
     */
	public $montant = 0;

	/**
     * Active
     * @var string
     */
	public $active = 1;

	/**
     * Transporter id
     * @var string
     */
	public $id = 0;


	/**
	 *  Constructor
	 *
	 *  @param      DoliDB		$db      Database handler
	 */
	function __construct($db)
	{
		global $langs, $conf;

		if ($conf->global->TRANSPORTER_ZONES_MODE == 'departements') {
			$this->table_element = 'c_tranches_poids_departements';
		} else if ($conf->global->TRANSPORTER_ZONES_MODE == 'regions') {
			$this->table_element = 'c_tranches_poids_regions';
		} else {
			$this->table_element = 'c_tranches_poids_pays';
		}

		$this->db = $db;
	}

    /**
     *    Check properties of product are ok (like name, barcode, ...).
     *    All properties must be already loaded on object (this->barcode, this->barcode_type_code, ...).
     *
     *    @return     int		0 if OK, <0 if KO
     */
    function verify()
    {
        $this->errors=array();

        $result = 0;

        return $result;
    }

	/**
	 *	Insert product into database
	 *
	 *	@param	User	$user     		User making insert
	 *  @param	int		$notrigger		Disable triggers
	 *	@return int			     		Id of product/service if OK, < 0 if KO
	 */
	function create($user,$notrigger=0)
	{
		global $conf, $langs, $mysoc;

        $error=0;

		
		dol_syslog(get_class($this)."::create", LOG_DEBUG);

		$now=dol_now();

		$this->db->begin();

		// Check more parameters
		// If error, this->errors[] is filled
		$result = $this->verify();

		if ($result >= 0)
		{

			$sql = "INSERT INTO ".MAIN_DB_PREFIX.$this->table_element." (";
			$sql.= " datec";
            $sql.= ", tms";
			$sql.= ", entity";
			$sql.= ", fk_transporteur";
			$sql.= ", fk_zone";
			$sql.= ", poids_min";
			$sql.= ", poids_max";
			$sql.= ", fk_unit_min";
			$sql.= ", fk_unit_max";
            $sql.= ", poids_reel";
			$sql.= ", montant";
			$sql.= ") VALUES (";
			$sql.= "'".$this->db->idate($now)."'";
            $sql.= ", '".$this->db->idate($now)."'";
            $sql.= ", ".$conf->entity;
			$sql.= ", ".$this->fk_transporteur;
			$sql.= ", ".$this->fk_zone;
			$sql.= ", ".$this->poids_min;
			$sql.= ", ".$this->poids_max;
			$sql.= ", ".$this->fk_unit_min;
			$sql.= ", ".$this->fk_unit_max;
            $sql.= ", ".$this->poids_reel;
            $sql.= ", ".$this->montant;
			$sql.= ")";

			dol_syslog(get_class($this)."::Create", LOG_DEBUG);
			$result = $this->db->query($sql);
			if ( $result )
			{
				$id = $this->db->last_insert_id(MAIN_DB_PREFIX.$this->table_element);

				if ($id > 0)
				{
					$this->id = $id;

				}
				else
				{
					$error++;
					$this->error='ErrorFailedToGetInsertedId';
				}
			}
			else
			{
				$error++;
				$this->error=$this->db->lasterror();
			}
				

			if (! $error)
			{
				$this->db->commit();
				return $this->id;
			}
			else
			{
				$this->db->rollback();
				return -$error;
			}
        }
        else
       {
            $this->db->rollback();
            dol_syslog(get_class($this)."::Create fails verify ".join(',',$this->errors), LOG_WARNING);
            return -3;
        }

	}

	/**
	 *	Update a record into database.
	 *
	 *	@param	int		$id         Id of product set
	 *	@param  User	$user       Object user making update
	 *	@param	int		$notrigger	Disable triggers
	 *	@param	string	$action		Current action for hookmanager ('add' or 'update')
	 *	@return int         		1 if OK, -1 if ref already exists, -2 if other error
	 */
	function update($id, $user, $notrigger=false, $action='update')
	{
		global $langs, $conf, $hookmanager;

		$error=0;
		


        $this->db->begin();

        // Check name is required and codes are ok or unique.
        // If error, this->errors[] is filled
        $result = 0;
        if ($action != 'add')
        {
        	$result = $this->verify();	// We don't check when update called during a create because verify was already done
        }

        if ($result >= 0)
        {
			$sql = "UPDATE ".MAIN_DB_PREFIX.$this->table_element;
			$sql.= " SET fk_transporteur = " . $this->fk_transporteur;
			$sql.= " , fk_zone = " . $this->fk_zone;
			$sql.= " , poids_min = " . $this->poids_min;
			$sql.= " , poids_max = " . $this->poids_max;
			$sql.= " , fk_unit_min = " . $this->fk_unit_min;
			$sql.= " , fk_unit_max = " . $this->fk_unit_max;
			$sql.= " , montant = " . $this->montant;
            $sql.= " , poids_reel = " . $this->poids_reel;
			$sql.= " , active = " . $this->active;
			$sql.= " WHERE rowid = " . $id;

			dol_syslog(get_class($this)."::update", LOG_DEBUG);

			$resql=$this->db->query($sql);
			if ($resql)
			{
				$this->id = $id;

				$this->db->commit();
				return 1;
			}
			else
			{
				$this->error=$langs->trans("Error")." : ".$this->db->error()." - ".$sql;
				$this->errors[]=$this->error;
				$this->db->rollback();

				return -2;				
			}           			


			if (! $error)
			{
				$this->db->commit();
				return $this->id;
			}
			else
			{
				$this->db->rollback();
				return -$error;
			}

        }
        else
        {
            $this->db->rollback();
            dol_syslog(get_class($this)."::Update fails verify ".join(',',$this->errors), LOG_WARNING);
            return -3;
        }
	}

	/**
	 *  Delete a product set from database (if not used)
	 *
	 *	@param      User	$user       Product id (usage of this is deprecated, delete should be called without parameters on a fetched object)
	 *  @param      int     $notrigger  Do not execute trigger
	 * 	@return		int					< 0 if KO, 0 = Not possible, > 0 if OK
	 */
	function delete(User $user, $notrigger=0)
	{
		global $conf, $langs;

		$error=0;

		// Check parameters
		if (empty($this->id))
		{
			$this->error = "Object must be fetched before calling delete";
			return -1;
		}
		
		$this->db->begin();

		// Delete product
		if (! $error)
		{
			$sqlz = "DELETE FROM ".MAIN_DB_PREFIX.$this->table_element;
			$sqlz.= " WHERE rowid = ".$this->id;
			dol_syslog(get_class($this).'::delete', LOG_DEBUG);
			$resultz = $this->db->query($sqlz);

			if ( ! $resultz )
			{
				$error++;
				$this->errors[] = $this->db->lasterror();
			}
		}

		

		if (! $error)
		{
			$this->db->commit();
			return 1;
		}
		else
		{
			foreach($this->errors as $errmsg)
			{
				dol_syslog(get_class($this)."::delete ".$errmsg, LOG_ERR);
				$this->error.=($this->error?', '.$errmsg:$errmsg);
			}
			$this->db->rollback();
			return -$error;
		}

	}

	/**
	 *  Load a product set in memory from database
	 *
	 *  @param	int		$id      			Id of product/service to load
	 *  @param  string	$ref     			Ref of product/service to load
	 *  @return int     					<0 if KO, 0 if not found, >0 if OK
	 */
	function fetch($id='')
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::fetch id=".$id);

		// Check parameters
		if (! $id)
		{
			$this->error='ErrorWrongParameters';
			dol_print_error(get_class($this)."::fetch ".$this->error);
			return -1;
		}

		$sql = "SELECT rowid, entity, fk_transporteur, fk_zone, poids_min, poids_max, fk_unit_min, fk_unit_max, montant, poids_reel, active, datec, tms";
		$sql.= " FROM ".MAIN_DB_PREFIX.$this->table_element;
		$sql.= " WHERE rowid = ".$this->db->escape($id);

		$resql = $this->db->query($sql);
		if ( $resql )
		{
			if ($this->db->num_rows($resql) > 0)
			{
				$obj = $this->db->fetch_object($resql);

				$this->id							= $obj->rowid;
				
				$this->fk_transporteur				= $obj->fk_transporteur;
				$this->fk_zone						= $obj->fk_zone;
				$this->poids_min					= $obj->poids_min;
				$this->poids_max					= $obj->poids_max;
				$this->fk_unit_min					= $obj->fk_unit_min;
				$this->fk_unit_max					= $obj->fk_unit_max;
				$this->montant						= $obj->montant;

				$this->active                       = $obj->active;
				$this->poids_reel                   = $obj->poids_reel;
				$this->date_creation				= $obj->datec;
				$this->date_modification			= $obj->tms;
				
				$this->entity						= $obj->entity;

				$this->db->free($resql);

				return 1;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			dol_print_error($this->db);
			return -1;
		}
	}

	/**
	 *  Load a product set in memory from database
	 *
	 *  @param	int		$id      			Id of product/service to load
	 *  @param  string	$ref     			Ref of product/service to load
	 *  @return int     					<0 if KO, 0 if not found, >0 if OK
	 */
	function getMinMaxForTransporters()
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::getMinMaxForTransporters");

		$maxs = array();
		$mins = array();

		$sql = "SELECT rowid, fk_transporteur, fk_zone, poids_min, fk_unit_min, poids_max, fk_unit_max, poids_reel, montant";
		$sql.= " FROM ".MAIN_DB_PREFIX.$this->table_element;
		$sql.= " WHERE entity IN (".getEntity($this->element).")";

		$resql = $this->db->query($sql);
		if ( $resql )
		{
			if ($this->db->num_rows($resql) > 0)
			{
				while ($obj = $this->db->fetch_object($resql))
				{
					$m = new stdClass();					
					$m->fk_transporteur				= $obj->fk_transporteur;
					$m->poids_min					= WeightTool::toKg($obj->poids_min, $obj->fk_unit_min);
					$m->poids_max					= WeightTool::toKg($obj->poids_max, $obj->fk_unit_max);
					$m->montant						= $obj->montant;
					$m->poids_reel                  = $obj->poids_reel;
					$m->id						= $obj->rowid;

					if (isset($mins[$m->fk_transporteur]))
					{
						if ($m->poids_min < $mins[$m->fk_transporteur]->poids_min)
						{
							$mins[$m->fk_transporteur] = $m;
						}
					}
					else
					{
						$mins[$m->fk_transporteur] = $m;
					}

					if (isset($maxs[$m->fk_transporteur]))
					{
						if ($m->poids_max > $maxs[$m->fk_transporteur]->poids_max)
						{
							$maxs[$m->fk_transporteur] = $m;
						}
					}
					else
					{
						$maxs[$m->fk_transporteur] = $m;
					}

				}


				$this->db->free($resql);
			}
		}


		return array($mins, $maxs);

	}	

	/**
	 *  Load a product set in memory from database
	 *
	 *  @param	int		$id      			Id of product/service to load
	 *  @param  string	$ref     			Ref of product/service to load
	 *  @return int     					<0 if KO, 0 if not found, >0 if OK
	 */
	function getRangesForZone($fk_zone = 0)
	{
		global $langs, $conf;

		dol_syslog(get_class($this)."::getRangesForZone");

		$ranges = array();

		$sql = "SELECT rowid, entity, fk_transporteur, fk_zone, poids_min, poids_max, fk_unit_min, fk_unit_max, poids_reel, montant, datec, tms";
		$sql.= " FROM ".MAIN_DB_PREFIX.$this->table_element;
		$sql.= " WHERE active = 1 AND entity IN (".getEntity($this->element).")";
		$sql.= " AND fk_zone =  ".(int)$fk_zone;
		
		$resql = $this->db->query($sql);
		if ( $resql )
		{
			if ($this->db->num_rows($resql) > 0)
			{
				while ($obj = $this->db->fetch_object($resql))
				{
					$range = new stdClass();

					$range->fk_transporteur = $obj->fk_transporteur;
					$range->fk_zone = $obj->fk_zone;
					$range->poids_min = WeightTool::toKg($obj->poids_min, $obj->fk_unit_min);
					$range->poids_max = WeightTool::toKg($obj->poids_max, $obj->fk_unit_max);
					$range->montant	 = $obj->montant;
					$range->poids_reel = $obj->poids_reel;
					$range->id	 = $obj->rowid;

					$ranges[$range->fk_transporteur][] = $range;
				}


				$this->db->free($resql);
			}
		}
		else
		{
			$this->error = $this->db->lasterror();
		}

		return $ranges;

	}	
}
